/*
Programm zur Ansteuerung eines I2C-Displays Sanbum LBL-11337, Pollin Bestell-Nr: 121449
2A im Sleep-mode, 50 A im Normalbetrieb Stromverbrauch des Displays
1mm Rasterma. Flexprintbuchse erforderlich zum Anschlu


Lesen des TWSR TWI-Statusregisters:
	return (TWSR & 0b11111100);	Man mu nur die letuzen zwei Bits auf Null setzen und NICHT drei nach rechts schieben!!! die Dritte Stelle ist immer Null.
	Deshalb ergeben sich immer Zahlen grer als mindestens 8.

Reset-Pin des Sanbum an + 3,3V anschlieen. Im Betrieb erzeigt das display selbst aus der Ladungspumpe 7,2V am Elko an Pins 1 und 2.


Version3:
Das Lesen aus dem Status-register geht jetzt inzwischen tatschlich. 
Das Display-Enable-Flag kann man setzen und erkennt danach eine nderung im Statusregister.

Bytes schreiben aufs Display geht und sie erscheinen als formen auf dem Display. Ebenso kann man Lschen durch vielmaliges Schreiben von 0x00.
Display schreibt von Linksunten nach rechts, dann die nchste Zeile obendrber wieder von links nach rechts, bis es oben rechts zu Ende ist.
Lschen geht selbst mit 	TWBR = 0x40;	=	55,5 kHz  ziemlich schnell

Version4:
Es soll gezielt an bestimmte Stellen aufs Display geschrieben werden unter Benutzung von "set page adresss" und "set column adress".
Es erscheinen aus dem Font die Zahlen, die als String geschrieben wurden, leider auf dem Kopf stehend. Unterprogramm vom Handydisplay bernommen und nur den Adressbefehl gendert.
Nimmt man den MSB-Font, stehen die Zeichen richtig herum!!! Halt schon mal bis ber 11000 durchgezhlt mit 	TWBR = 0x80 und 	CLKPR = 1 << CLKPS0  = 4 MHz 

Version5:
Des selben String zwei mal schreiben an unterscheidliche Positionen und schauen, ob man unntige Befehle auslassen kann. Ja, kann man weg lassen. 
Man braucht nur die neue Position angeben in einem extra Kommandozyklus und dann den String ausgeben.

Version6:
Nochmals testen, wie die Zeilen aufgebaut sind, wie man Lscht und wie Ausgaben zu positionieren sind.
Eine Zeile ist genau 131 Pixel breit: for ( uint16_t ijk = 0; ijk < 132; ijk++)	//bis < 132 ist genau eine Zeile breit
Man kann 64 Zeilen weit nach unten Scrollen, um wieder das gleiche Bild zu erhalten.  Das Display hat 21 Pixels Hhe.
Nach drei Bildschirmen ( 3 x 21 = 63 ) erscheint wieder das selbe Bild. 64 / 8 = 8 Display hat also 8 Zeilen!!! von denen man 5 nicht sieht.
131 Spalten * 8 Zeilen = 1048	Das Lschen mte 1048 Bytes mit 0x00 schreiben.

Version7:
Es soll ein Pixel an eine beliebige x-y-Koordinate geschrieben werdenknnen mittels Funktion "Setpixel". Hierzu zuerst positionieren, 
dann ein Byte an genau dieser Stelle lesen, dieses mit dem Pixel verunden oder zum Lschen verodern 
und dann das neue Byte wieder genau an diese Stelle schreiben. Von Sanbum_display_SetPixel bernommen.

AVR Memory Usage				mit der ersten Hlfte (erste 64 Zeichen) des 8x8 vertikal_LSB Fonts, macht 512 Bytes mehr aus
----------------
Device: atmega48

Program:    3162 bytes (77.2% Full)
(.text + .data + .bootloader)

Data:         30 bytes (5.9% Full)
(.data + .bss + .noinit)


*/



#include <avr/io.h>
#include <stdlib.h>				// wird nur fr die Funktion "iota" bei der LCD-Ausgabe gebraucht.
#include <avr/pgmspace.h>		//zum Eintragen des Textes, der die Programmversion anzeigt in der hex-Datei
#include "sanbum.h"
#include <util/delay.h>
//#include "8x8_vertikal_LSB_1.h"	//stehen auf dem Kopf
#include "8x8_vertikal_MSB_1.h"		//stehen richtig herum, Kabel zeigt zum Betrachter



//#define read  			1
#define read  			0x01
#define write  			0


/* gehrt zum Sanbum-Display */
#define Adresse_control		0x38		//Control-Adresse	Sanbum-I2C-Display
#define Adresse_data		0x39		//Daten				Sanbum-I2C-Display
#define kontrast 			0xC0		//Kontrast Wert ausprobieren, 0xC0 ist default


/* Display-Kommandos A0=low*/
#define col_low        0
#define col_hi      0x10
#define temp_comp   0x24
#define pwr_ctrl    0x28
#define adv_pctl    0x30   // danach Wert ?
#define scrolline   0x40
#define set_page    0xB0
#define poti        0x81   // danach Wert, default 0xC0
#define partial     0x84
#define ramadctl    0x88
#define framerate   0xA0
#define allpixel    0xA5	//mit 0xA4 sind Pixel aus
#define invers      0xA6
#define disp_ena    0xAF	//mit 0xAE ist Display aus
#define map_ctrl    0xC0
#define reset       0xE2
#define nop         0xE3
#define bias        0xE8	// Bias-ratio = 6 ist der default-Wert. Damit lt sich das Display "ganz dunkel schwarz" steuern
#define com_end     0xF1    // danach Wert 0..127, default 63	//COM_end einstellen: 0x15 = 21. Das Display hat also 21 Pixels Hhe
#define part_start  0xF2    // danach Wert 0..127, default 0
#define part_end    0xF3    // danach Wert 0..127, default 63



const char str0[] PROGMEM = "Sambum_Mega48";


/* gehrt zum Sanbum-Display */
const uint8_t UC1601_init[13] = {		//die sieben Bytes zum Initialisieren des Displays von hier auslesen und ans Display bertragen.
0x24,//|0b00,		//Set Temperature Comp. TC[1:0]
// falsch 0x28,//|0b110,		//Set Power Control PC[2:0]		bedeutet internal VLCD x 7
0x2C,//|0b110,		//Set Power Control PC[2:0]		bedeutet internal VLCD x 7
0x40,//|0b000000,	//Set Scroll Line SL[5:0]	
//0x81, gnstiger Wert	//Set VBIAS Potentiometer PM [7:0] (Double byte command)
0x81,					//bei 0x50 begint das Display grau zu werden. Bei 0x80 sind Punkte schwarz. Set VBIAS Potentiometer PM [7:0] (Double byte command)
0xC0,		
0x84,//|0b0,		//Set Partial Display Enable LC [4]
0x88,//|0b001,		//Set RAM Address Control AC [2:0]
0xA0,//|0b0 ,		//Set Frame Rate LC [3]
0xC0,//|0b100,		//Set LCD Control LC[2:1]
0xE8,//|0b00, 		//Set Bias Ratio BR [1:0]	Bias-ratio = 6 ist der default-Wert. Damit lt sich das Display "ganz dunkel schwarz" steuern
0xF1,				//Set CEN [6:0] (Double byte command)
0x15,				///auf diesen Wert COM_end einstellen: 0x15 = 21. Das Display hat also 21 Pixels Hhe
0xAF//|0b1			//Set Display Enable DC[2]		0xAe = off, 0xAF = on
};


//uint16_t zaehlvariable = 0;
uint16_t zaehlvariable = 0;
uint8_t zaehlvariable2 = 0;
uint8_t farbe = 1;
char s[10];		//Array fr ITOA

uint8_t	ext_statusbyte1 = 0;
uint8_t ext_statusbyte2 = 0;		//Versuch, zwei statusbytes nacheinander zu lesen. vielelicht sind sie unterschiedlich
uint8_t	ext_statusbyte3 = 0;


uint8_t Sanbum_send_start (void)
{
	/*writing a one to TWINT clears it, TWSTA=Start, TWEN=TWI-enable*/
	TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);
	/*wait, until start condition has been sent --> ACK*/
	while (!(TWCR & (1<<TWINT)));

	return (TWSR);
//	return (TWSR & 0b11111100);

}


void Sanbum_send_stop (void)
{
	/*writing a one to TWINT clears it, TWSTO=Stop, TWEN=TWI-enable*/
	TWCR = (1<<TWINT) | (1<<TWSTO) | (1<<TWEN);	
}


/* gehrt zum Sanbum */
uint8_t Sanbum_read_byte (void)		//liest ein Byte vom Display, erste Erprobung mit dem Statusbyte, Master sendet Ack
{								//OK, geht!!
	/*send content of TWDR; TWEA = enable ACK*/
	TWCR = (1<<TWINT) | (1<<TWEA) | (1<<TWEN);
	/*wait, until byte has been received --> ACK*/
	while (!(TWCR & (1<<TWINT)));
	
	return TWDR;
//	return (TWSR & 0b11111100);

}


uint8_t Sanbum_send_add_rw (uint8_t addr_byte, uint8_t rw)
{
	/*address can be 0 .. 8; rw=0 --> write, rw=1 --> read*/

/*******************************/
	/*shift address one bit left*/
	addr_byte = addr_byte << 1;				//diesen Befehl weg lassen oder dazu tun, Adresse mu um 1 nach links verschoben werden laut Bild Seite 27/31
/*******************************/

	
	/*set RW-Bit, if necessary*/
	addr_byte |= rw;

	/*TWDR contains byte to send*/
	TWDR = addr_byte;
	
	
	/*send content of TWDR*/
	TWCR = (1<<TWINT) | (1<<TWEN);
	/*wait, until address has been sent --> ACK*/
	while (!(TWCR & (1<<TWINT)));

	return TWSR;
//	return (TWSR & 0b11111100);

}


uint8_t Sanbum_send_byte (uint8_t data_byte)	//sendet ein Daten-Byte
{
	TWDR = data_byte;

	TWCR = (1<<TWINT) | (1<<TWEN);
	/*wait, until address has been sent --> ACK*/
	while (!(TWCR & (1<<TWINT)));

	return TWSR;
//	return (TWSR & 0b11111100);

}



void LCD_Softreset (void)
{
//uint8_t	status_ausgabe = 0;
	Sanbum_send_start();
//	status_ausgabe = Sanbum_send_start();		// -->> 48		-->> Data byte has been transmitted;NOT ACK has been received
	

	
	Sanbum_send_add_rw (Adresse_control,write);	//zuerst die Adresse und Schreiben senden.  
//	status_ausgabe = Sanbum_send_add_rw (Adresse_control,write);	//-->> 48


	Sanbum_send_byte(reset);				//Kommando "reset" geben.
//	status_ausgabe = Sanbum_send_byte(reset);				//Kommando "reset" geben.  -->> 48
	

	Sanbum_send_stop();
	_delay_ms(10);
}



void LCD_all_on (void)
{
//hiermit "all Pixels on" bestimen.	
	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_control,write);	//Adresse geben und Schreiben bestimmen
	Sanbum_send_byte(allpixel);
	Sanbum_send_stop();	
}

/*
void LCD_enable (void)				//wird aufgerufen von "sanbum_init"
{

	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_control,write);	//zuerst die Adresse und Schreiben senden.  
	Sanbum_send_byte(disp_ena);	  				//Kommando "set_display_enable" geben. es bentigt anscheinend keinen Wert danach.
	Sanbum_send_stop();
	_delay_ms(10);	
	
}
*/
void LCD_enable (void)				//wird aufgerufen von "sanbum_init"
{

	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_control,write);	//zuerst die Adresse und Schreiben senden.  

	
/************************************/	
	uint8_t i;
	for(i=0;i<13;i++) 
	{

	Sanbum_send_byte(UC1601_init[i]);		//nacheinander die Bytes aus dem Array ins Display schreiben als Kommandos
	
	}
/************************************/	
	Sanbum_send_byte(disp_ena);	  				//Kommando "set_display_enable" geben. es bentigt anscheinend keinen Wert danach.
	Sanbum_send_stop();
	_delay_ms(10);	
	
}


void LCD_SetComEnd (void)			//wird aufgerufen von "sanbum_init"			siehe Datenblatt, Seite 15/19 "set com end"
{
	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_control,write);	//zuerst die Adresse und Schreiben senden.
	Sanbum_send_byte(com_end);					//Kommando "com_end" bestimmen
	Sanbum_send_byte(63);						//"com_end" soll 63 sein, also hier den Wert angeben
	Sanbum_send_stop();
	_delay_ms(10);
}


void LCD_SetKontrast (void)			//wird aufgerufen von "sanbum_init"
{
	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_control,write);	//zuerst die Adresse und Schreiben senden.
	Sanbum_send_byte(poti);						//Kommando "poti" bestimmen
	Sanbum_send_byte(kontrast);					//Wert angeben, poti soll defaultwert 0xC0 sein
	Sanbum_send_stop();
	_delay_ms(10);
}



void sanbum_init (void)
{
	LCD_Softreset();
	LCD_SetKontrast();
	LCD_SetComEnd();
	LCD_enable();
//	LCD_all_on();
}


//gehrt zum Sanbum
uint8_t read_statusbyte (void)		//hierfr mu Kommando/Daten-Bit auf 0 sein! -->> 0x39 die Daten-Adresse lesen!
{
	uint8_t statusbyte;
	Sanbum_send_start();

	Sanbum_send_add_rw (Adresse_control,read);		//0x38 , lesen (des Statusbytes)	
										//-->>	add_rw: 0x40	SLA+R has been transmitted; ACK has been received
										//-->>	readbyte: 0x50	Data byte has been received; ACK has been returned
//	statusbyte = Sanbum_read_byte();
	ext_statusbyte1 = Sanbum_read_byte();	//zum ersten Mal Lesen und in erster Variablen speichern
//	ext_statusbyte2 = Sanbum_read_byte();	//zum zweiten Mal lesen und in anderer Variablen speichern

	Sanbum_send_stop();
	return statusbyte;
}


//gehrt zum Sanbum
uint8_t read_RAM_Byte (void)	//liest ein Byte aus dem Display-RAM. Zuvor mu man die Position einstellen, an der gelesen werden soll.
{							//hierfr mu Kommando/Daten-Bit auf 1 sein! -->> 0x39 die Daten-Adresse lesen!
	uint8_t byte;
//	I2Clcd_string("sendstart: ");
	Sanbum_send_start();

	Sanbum_send_add_rw (Adresse_data,read);		//0x39

	byte = Sanbum_read_byte();

	Sanbum_send_stop();
	return byte;
}

void Sanbum_GoTo (uint8_t x, uint8_t y)	//Cursor auf eine beliebige x-y-Koordinate auf der Displayflche stezen, kann auch im unsichtbaren Bereich sein, den man nur durch Scrollen sieht.
{										//Sanbum hat eine Displayflche von x = 131 Pixeln und y = 63 Pixeln
	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_control,write);	//Adresse geben und Schreiben bestimmen
//	Sanbum_send_byte(ramadctl | 0x01);		//Kommando "RAM-Adress-Control" = 0x88 geben und AC[0] = 1 setzen -->>  CA or PA (depends on AC[1]= 0 or 1) will restart, and CA or PA will increase by one. 
	Sanbum_send_byte(col_low | (x & 0x0F));//0..16 ist mglich		Obere 4 Bits sind der Befehl, untere 4 Bits sind die Position-unteres Nibble
	Sanbum_send_byte(col_hi | (x >> 4));	//0...16 ist mglich	Obere 4 Bits sind der Befehl. Die Bits von X um vier nach rechts schieben und mit dem Befehl verodern.
	Sanbum_send_byte(set_page | (y / 8));	//0..63 ist mglich		0 = unterste Zeile, 1 = mittlere Zeile, 3 = obere Zeile
	Sanbum_send_stop();	

}

// Sanbum_GLCD_SetPixel		//mit deaktiviertem Leseteil geht es schneller. Evtl ber Parameter einstellbar machen.
//-------------------------------------------------------------------------------------------------
void Sanbum_display_SetPixel(uint8_t x, uint8_t y, uint8_t color)  //color=1 => Pixel=Black  color = 0 -->> wei
{		
	uint8_t temp = 0;  

/*	Sanbum_GoTo(x, y); 	
	read_RAM_Byte();
	temp = read_RAM_Byte();						//save old state 
*/
	Sanbum_GoTo(x, y); 							//Chips increments automatically, jump back
	
	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_data,write);	//zuerst die Adresse, Daten auswhlen und Schreiben senden.	
	if(color)
	  Sanbum_send_byte(temp | (1 << (y % 8)));    
	else
	  Sanbum_send_byte(temp & ~(1 << (y % 8))); 
	Sanbum_send_stop();	
}
//-------------------------------------------------------------------------------------------------





/* gehrt zum Sanbum */
void Sanbum_display_clear (void)		//OK, funktioniert!!! Aber Programm bleibt manchmal danach stehen. warum???
{
	uint16_t	i = 0; //j = 0;
	
//Lschen soll nicht irgendwo beginnen, sondern immer in der unteren linken Ecke.	
	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_control,write);	//Adresse geben und Schreiben bestimmen
	Sanbum_send_byte(ramadctl | 0x01);			//Kommando "RAM-Adress-Control" = 0x88 geben und AC[0] = 1 setzen -->>  CA or PA (depends on AC[1]= 0 or 1) will restart, and CA or PA will increase by one. 
	Sanbum_send_byte(col_low | 0);				//0..16 ist mglich		mit 16 wird immer ein 16 Spalten breites Rechteck mit Bytes beschrieben von rechts nach  links und beginnt wieder am linken Rechteckrand.
	Sanbum_send_byte(col_hi | 0);				//0...16 ist mglich	deswegen Low immer auf 16 lassen, dann 16 Bytes schreiben, dann high um eins weiter schalten
	Sanbum_send_byte(set_page | 0);				//0..8 ist mglich		0 = unterste Zeile, 1 = mittlere Zeile, 3 = obere Zeile
	Sanbum_send_stop();	
	
	
	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_data,write);			//zuerst die Adresse und Schreiben senden.
	for(i=0;i<1048;i++) 
		{
		Sanbum_send_byte(0x00);		//schreibt in 8er Spalten ausgeschaltete Pixel aufs Display
		}
	Sanbum_send_stop();
}


//schreibt ein einzelnes Zeichen aus dem vorgegebenen vertiklalen Font 8x8. zum Sanbum GLCD
//-------------------------------------------------------------------------------------------------
void Sanbum_WriteChar8x8(char x)	//fr horizontale Adressierung
{
char i; 
//x -= 32; 	//32subtrahieren, dann zuweisen					//refers to the 8x8_vertikal_LSB_1.h library

	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_data,write);//zuerst die Adresse und Schreiben senden.

//for(i = 0; i < 8; i++) Sanbum_send_byte(pgm_read_byte(&font8x8_vertikal_LSB_1 [x][i]));	//stehen auf dem Kopf
for(i = 0; i < 8; i++) Sanbum_send_byte(pgm_read_byte(&font8x8_vertikal_MSB_1 [x][i]));	//stehen richtig herum, Kabel zeigt zum Betrachter

//Sanbum_send_byte(0x00);						//space between two char-signs

	Sanbum_send_stop();
}


//zum Sanbum GLCD
void Sanbum_WriteString8x8(char * s)	//fr horizontale Adressierung
{
while(*s)
  {
  Sanbum_WriteChar8x8(*s++);
  }
}


// Sanbum_GLCD_SetPixel
void Sanbum_Circle(uint8_t cx, uint8_t cy ,uint8_t radius)
{
int x, y, xchange, ychange, radiusError;
x = radius;
y = 0;
xchange = 1 - 2 * radius;
ychange = 1;
radiusError = 0;
while(x >= y)
  {
  Sanbum_display_SetPixel(cx+x, cy+y, farbe); 
  Sanbum_display_SetPixel(cx-x, cy+y, farbe); 
  Sanbum_display_SetPixel(cx-x, cy-y, farbe);
  Sanbum_display_SetPixel(cx+x, cy-y, farbe); 
  Sanbum_display_SetPixel(cx+y, cy+x, farbe); 
  Sanbum_display_SetPixel(cx-y, cy+x, farbe); 
  Sanbum_display_SetPixel(cx-y, cy-x, farbe); 
  Sanbum_display_SetPixel(cx+y, cy-x, farbe); 
  y++;
  radiusError += ychange;
  ychange += 2;
  if ( 2*radiusError + xchange > 0 )
    {
    x--;
	radiusError += xchange;
	xchange += 2;
	}
  }
}

/*
void Sanbum_Line(unsigned int X1,unsigned int Y1,unsigned int X2,unsigned int Y2)
{
int CurrentX, CurrentY, Xinc, Yinc, 
    Dx, Dy, TwoDx, TwoDy, 
	TwoDxAccumulatedError, TwoDyAccumulatedError;

Dx = (X2-X1); // obliczenie skadowej poziomej
Dy = (Y2-Y1); // obliczenie skadowej pionowej

TwoDx = Dx + Dx; // podwojona skadowa pozioma
TwoDy = Dy + Dy; // podwojona skadowa pionowa

CurrentX = X1; // zaczynamy od X1
CurrentY = Y1; // oraz Y1

Xinc = 1; // ustalamy krok zwikszania pozycji w poziomie 
Yinc = 1; // ustalamy krok zwikszania pozycji w pionie

if(Dx < 0) // jesli skadowa pozioma jest ujemna 
  {
  Xinc = -1; // to bdziemy si "cofa" (krok ujemny)
  Dx = -Dx;  // zmieniamy znak skadowej na dodatni
  TwoDx = -TwoDx; // jak rwnie podwojonej skadowej
  }

if (Dy < 0) // jeli skadowa pionowa jest ujemna
  {
  Yinc = -1; // to bdziemy si "cofa" (krok ujemny)
  Dy = -Dy; // zmieniamy znak skadowej na dodatki
  TwoDy = -TwoDy; // jak rwniez podwojonej skadowej
  }
*/

//aus bresenham.c	Zingl Alois	17.12.2012	Curve Rasterizing Algorithm
void Sanbum_plotLine(int x0, int y0, int x1, int y1)	//OK, funktioniert perfekt. Linie mit 1 Pixel Dicke.
{
   int dx =  abs(x1-x0), sx = x0<x1 ? 1 : -1;
   int dy = -abs(y1-y0), sy = y0<y1 ? 1 : -1;
   int err = dx+dy, e2;                                   // error value e_xy
                                                    
   for (;;){                                                          // loop
//		x_y_pixel(x0,y0);  
		Sanbum_display_SetPixel (x0, y0, farbe);                            
      e2 = 2*err;                                   
      if (e2 >= dy) {                                         // e_xy+e_x > 0
         if (x0 == x1) break;                       
         err += dy; x0 += sx;                       
      }                                             
      if (e2 <= dx) {                                         // e_xy+e_y < 0
         if (y0 == y1) break;
         err += dx; y0 += sy;
      }
   }
}

/*
//es erscheint nur eine Waagrechte, kann keine Diagonale zeichenen
void Sanbum_simple_line(unsigned char x, unsigned char y, unsigned char b, unsigned char a)
{
  unsigned char j;
  for (j = 0; j < b; j++) {
		Sanbum_display_SetPixel(x + j, y, farbe);
	}

}
*/


int __attribute__((OS_main)) main(void)
{

/*
Wie bekommt man die Oszillatorfrequenz von 8 MHz auf 4 MHz?
CLKPR CLKPCE    CLKPS3 CLKPS2 CLKPS1 CLKPS0
CLKPS3 CLKPS2 CLKPS1 CLKPS0 Clock division factor
	0 	0 		0 		0 			1
	0 	0 		0 		1			2			<<<<--- DIESEN VERWENDEN

The CLKPCE bit must be written to logic one to enable change of the CLKPS bits. The CLKPCE
bit is only updated when the other bits in CLKPR are simultaneously written to zero.
*/


//Takt einstellen am Mega48:  (bei 8 MHz sausen die Zahlen auf dem I2C-LCD einwandfrei ber 2 x 4,7kOhm zu +3,3 Volt)
//Fr extrem langsame Taktraten an den Programmstart ein delay_ms (5000) setzen, damit man vorher den Programmer starten kann laut Peter, ist beestimmt auch gefhrlich.

	CLKPR = 0x80;			//CLKPCE auf 1 setzen und alle anderen Bits auf 0, um die Funktionalitt zu aktivieren
//	CLKPR = 1 << CLKPS2;	//8 MHz / 16 = 500 kHz 	
	CLKPR = 1 << CLKPS0;	//8 MHZ / 2 = 4 MHz 		<<<---<<<<-- diese verwenden fr 4 MHz
//	CLKPR = 1 << CLKPS1;	//8 MHZ / 4 = 1 MHz 
//	CLKPR = (1 << CLKPS1) | ( 1<< CLKPS2 );	//  C kann danach nicht mehr angesprochen werden. NICHT verwenden!!! 8 MHZ / 64 =  125 kHz
												// bei 125 KHz mu der ISP-Takt in AVRdude verlangsamt werden mit -i 1000    ISP-Takt delay 1000s


	DDRD = 0xE0;	//als Ausgang schalten fr LEDs


	/*set I2C - bus speed*/	//TWBR soll nicht kleiner als 10 sein!!!
	TWBR = 0x10;	//	166 KHz	OK, 222 kHz mit 8 MHz, geht mit 2 x 4,7 K an 3,5 Volt, sehr schnell!! Auf Platine mit 3 x 28-Pin C
//	TWBR = 0x40;	//	55,5 kHz, geht mit 2 x 4,7 K an 3,5 Volt
//	TWBR = 0x50;	//	45,5 kHz, 
//	TWBR = 0x80;	// -->> 29 kHz, geht noch mit 18 kOhm - Widerstnden am Bus
	//fSCK = 8 MHz / (16 + 2 TWBR)	-->> 8000000 / (16 + 2 * 128) = 29411 Hz
	//fSCK = 4 MHz / (16 + 2 TWBR)	-->> 4000000 / (16 + 2 * 128) = 14705 Hz	


//	TWBR = 0xFF;	// -->> 7604 Hz			//langsamste Einstellung, ohne noch zustzliche Prescaler-Bits im TWSR zu setzen
	//fSCK = 4 MHz / (16 + 2 TWBR)	-->> 4000000 / (16 + 2 * 255) = 7604 Hz	

	
	//ab hier Versuche mit zwei 1,8 k Widerstnden am I2C-bus:
//	TWBR = 0x10;	//		166 KHz	OK, ist schnell und Bus luft stabil, aber RC5 reagiert fast nicht mehr.
//	TWBR = 0x04;	//		333 kHz
//	TWBR = 0x03;	//		364 kHz
//	TWBR = 0x02;	//		400 kHz	OK, ist schnell und Bus luft stabil, aber RC5 reagiert fast nicht mehr.
	


	sanbum_init();	
	
	Sanbum_display_clear();

    while (1) 
	{
	zaehlvariable++;
	zaehlvariable2++;
	PORTD = zaehlvariable;
	//Am Mega48 sind drei LEDs angeschlossen: grn PD5, gelb an PD6, rot an PD7
	

/*	
//Ausgabe einer Zahl mittels Font
	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_control,write);	//Adresse geben und Schreiben bestimmen
	Sanbum_send_byte(ramadctl | 0x01);			//Kommando "RAM-Adress-Control" = 0x88 geben und AC[0] = 1 setzen -->>  CA or PA (depends on AC[1]= 0 or 1) will restart, and CA or PA will increase by one. 
//	Sanbum_send_byte(ramadctl | 0x07);			//Kommando "RAM-Adress-Control" = 0x88 geben und AC[2] = 1 und AC[0] = 1 setzen	-->> schreibt nach oben
//	Sanbum_send_byte(ramadctl | 0x03);			//Kommando "RAM-Adress-Control" = 0x88 geben und AC[1] = 1 und AC[0] = 1 setzen	
//	Sanbum_send_stop();							//Dies kann auch in der Initialisierung erfolgen.


//zustzlich Befehl "set column adress" anwenden. Dies ist ein 2-Byte Befehl. Im ersten Byte werden die unteren 4 Bits bertragen, im zweiten die oberen 4 Bits.
//	Sanbum_send_start();
//	Sanbum_send_add_rw (Adresse_control,write);	//Daten und Schreiben bestimmen
	Sanbum_send_byte(col_low | 0);				//0..16 ist mglich		mit 16 wird immer ein 16 Spalten breites Rechteck mit Bytes beschrieben von rechts nach  links und beginnt wieder am linken Rechteckrand.
	Sanbum_send_byte(col_hi | 0);				//0...16 ist mglich	deswegen Low immer auf 16 lassen, dann 16 Bytes schreiben, dann high um eins weiter schalten
	Sanbum_send_byte(set_page | 0);				//0..8 ist mglich		0 = unterste Zeile, 1 = mittlere Zeile, 3 = obere Zeile
	Sanbum_send_stop();	

	Sanbum_WriteString8x8(s);		//schreibt die Zahl, die zuvor durch ITOA in den String kopiert wurde.
*/

/*
//hiermit mal einfach SCHREIBEN, um das Display-RAM mit Werten zu fllen. Mal sehen, welcher Pixelbrei erscheint.
//Hierfr nur auf die DATEN-Adresse schreiben! Ja, das geht, aber nach einer Zeile bleibt es stehen.
	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_control,write);	//Adresse geben und Schreiben bestimmen
	Sanbum_send_byte(ramadctl | 0x01);			//Kommando "RAM-Adress-Control" = 0x88 geben und AC[0] = 1 setzen -->>  CA or PA (depends on AC[1]= 0 or 1) will restart, and CA or PA will increase by one. 
	Sanbum_send_byte(col_low | 0);				//0..16 ist mglich		mit 16 wird immer ein 16 Spalten breites Rechteck mit Bytes beschrieben von rechts nach  links und beginnt wieder am linken Rechteckrand.
	Sanbum_send_byte(col_hi | 0);				//0...16 ist mglich	deswegen Low immer auf 16 lassen, dann 16 Bytes schreiben, dann high um eins weiter schalten
	Sanbum_send_byte(set_page | 0);				//0..8 ist mglich		0 = unterste Zeile, 1 = mittlere Zeile, 3 = obere Zeile
	Sanbum_send_stop();	


//Mit AC [0] = 0 wird nur in die aktuelle Zeile geschrieben und dann hrt es auf.
	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_data,write);	//Daten und Schreiben bestimmen
//	Sanbum_send_byte(zaehlvariable);
//	for ( uint16_t ijk = 0; ijk < 132; ijk++)	//bis < 132 ist genau eine Zeile breit
//	for ( uint16_t ijk = 0; ijk < 264; ijk++)	//bis < 264 ist genau zwei Zeilen breit
	for ( uint16_t ijk = 0; ijk < 396; ijk++)	//bis < 396 ist genau drei Zeilen breit
		{	
//		Sanbum_send_byte(ijk);
		Sanbum_send_byte(10);
		 _delay_ms ( 60 );
		}
	Sanbum_send_stop();
*/	
//	Sanbum_display_clear();


/*
//Kommando "scroll_line ausprobieren" ************************
//Nach 64 ist das bild einmal nach unten raus durchgelaufen und sieht wieder so aus wie zu Beginn ohne Scrollen.
	Sanbum_send_start();
	Sanbum_send_add_rw (Adresse_control,write);	//Adresse geben und Schreiben bestimmen
	Sanbum_send_byte(ramadctl | 0x01);			//ohne schreibt es nur in eine Zeile, mit | 1 schreibt es in alle Zeilen obendrber	
	Sanbum_send_byte(scrolline | (zaehlvariable & 0b00111111));			//		0 bis 63 maximal		//schiebt die Schrift nach unten aus dem Display heraus.
//	Sanbum_send_byte(scrolline | 62);			//	0 bis 64 maximal, schiebt die Schrift nach unten aus dem Display heraus. Bei 64 wieder wie original
	Sanbum_send_stop();	
	_delay_ms ( 300 );
*/


//Positionierung in die einzelnen Zeilen funktioniert und die Spalten kann man sowieso direkt einstellen
//	Sanbum_GoTo (20, 12);	// x = 0...131, y = 0...63
//	Sanbum_WriteString8x8("Sanbum");//schreibt einen Text

	itoa(zaehlvariable, s, 10);


	Sanbum_GoTo (40, 8);	// x = 0...131, y = 0...63
	Sanbum_WriteString8x8(s);		//schreibt die Zahl, die zuvor durch ITOA in den String kopiert wurde.

	
//	Sanbum_display_SetPixel ( zaehlvariable+ 12, zaehlvariable, farbe );	// x, y, color		zeigt eine steigende Diagonale
//	Sanbum_Circle (90, 10, 8);	//x, y, radius		//Kreis ist OK
	Sanbum_Circle (90, 10, zaehlvariable2);	//x, y, radius		//Kreis ist OK
//	Sanbum_simple_line (10, 3, 80, 20);	//ergibt eine waagrechte Gerade
//	Sanbum_plotLine (10, 3, 80, 20);	//funktioniert!
	Sanbum_plotLine (0, 0, zaehlvariable, 21);	//funktioniert! Es gibt einen Unterschied, ob "Setpixel" vorher liest oder nicht.


	if (zaehlvariable2 > 21) zaehlvariable2 = 0;

//	if ( (zaehlvariable & 0x1F) > 21 ) {farbe = 0; zaehlvariable = 0;}			//Umschalten des Pixels zwischen hell und dunkel nach einem Durchlauf
//	if (zaehlvariable == 20) {Sanbum_display_clear(); zaehlvariable = 0; 	_delay_ms ( 200 );	}
	_delay_ms ( 200 );	
	
    }

    return 0;
}
